import {Component, Input, OnInit} from '@angular/core';
import {ProductGamesystem} from "../../../../game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../../../game-model/gamesystems/SimpleGamesystem";
import {
  MatCell, MatCellDef,
  MatColumnDef, MatHeaderCell, MatHeaderCellDef,
  MatHeaderRow,
  MatHeaderRowDef,
  MatRow,
  MatRowDef,
  MatTable,
  MatTableDataSource
} from "@angular/material/table";
import {SimpleState} from "../../../../game-model/gamesystems/SimpleState";
import {State} from "../../../../game-model/gamesystems/State";
import {NgForOf} from "@angular/common";
import {ProductState} from "../../../../game-model/gamesystems/ProductState";

@Component({
  selector: 'app-product-state-editor',
  standalone: true,
  imports: [
    MatTable,
    MatRow,
    MatRowDef,
    MatHeaderRow,
    MatHeaderRowDef,
    MatColumnDef,
    MatHeaderCell,
    MatHeaderCellDef,
    MatCell,
    MatCellDef,
    NgForOf
  ],
  templateUrl: './product-state-editor.component.html',
  styleUrl: './product-state-editor.component.scss'
})
export class ProductStateEditorComponent implements OnInit{

  @Input() gamesystem: ProductGamesystem | undefined
  displayedColumns: string[] = [];
  datasource = new MatTableDataSource();


  ngOnInit() {
    this.gamesystem!.generateFromChildsystems();
    this.generateColumnNamesRecursively(this.gamesystem!, "");
    this.datasource.data = this.gamesystem!.states;
  }

  generateColumnNamesRecursively(gamesystem: ProductGamesystem, nestedColumnName: string) {
    gamesystem.innerGamesystems.forEach(innerGamesystem => {
      if(innerGamesystem instanceof SimpleGamesystem) {
        this.displayedColumns.push(nestedColumnName + innerGamesystem.componentName);
      } else {
        this.generateColumnNamesRecursively(innerGamesystem as ProductGamesystem, nestedColumnName + innerGamesystem.componentName + ".");
      }
    })
  }

  protected readonly SimpleState = SimpleState;

  getStateLabel(state: State<any>, i: number) {
    return this.computeLeafStates(state)[i].stateLabel;
  }

  computeLeafStates(state: State<any>) {
    if(state instanceof SimpleState) {
      return [state];
    } else {
      const productState = state as ProductState;
      const leafStates: SimpleState[] = [];
      productState.innerStates.forEach(innerState => {
        this.computeLeafStates(innerState).forEach(leafState => leafStates.push(leafState));
      })
      return leafStates;
    }
  }
}
